 %Kostyantyn Partola
%FRAME Data Loader v1.6

%preparing variables for a new run
currentcurve=1;

%getting the input path for the file
%Foldpath=get(handles.CurvesPath,'String');
Foldpath=uigetdir;
set(handles.Loading_Text,'String','Reading Data')
drawnow
IBWlist=getAllFiles(Foldpath);  %this function can be found on Mathworks
n=0;
FCnumber='0';

%scanning through each file
for k=1:length(IBWlist)
    if wrev(IBWlist{k}(length(IBWlist{k}):-1:length(IBWlist{k})-3))=='.ibw'
        
        n=n+1;
        FCnumber=num2str(n);
        LoadingMessage=['Loading Curve ' FCnumber];
        set(handles.Loading_Text,'String',LoadingMessage)
        drawnow
        
        %Each wave is read by IBWread.m (this function is on Mathworks)
        Wbuff=IBWread(IBWlist{k});
        
        R=textscan(Wbuff.WaveNotes,'%s'); %seperating the wave note data
        
        %Reading the SpringConstant and the location of the reversal of
        %direction of the curve. This allows FRAME to know when the
        %extension ends and the retraction of the curve begins.
        SpcR=0;
        IndexR=0;
        Flip=0;
        X_ind=0;
        Y_ind=0;
        for spk=1:length(R{1,1})
            if strcmp(R{1,1}{spk},'SpringConstant:') && SpcR==0
                SpcR=spk+1;
            end
            
            if strcmp(R{1,1}{spk},'Indexes:') && IndexR==0;
                IndexR=spk+1;
                Flip=str2num(R{1,1}{IndexR});
            end
            
            if (length(R{1,1}{spk}) > 8) && IndexR == 0
                if strcmp(R{1,1}{spk}(1:8),'Indexes:')
                    IndexR = spk;
                    Flip = str2num(R{1,1}{IndexR}(9:end));
                end
            end
            
            %The data loader also reads the "X" and "Y" lvdt sensor in
            %order to produce topographical maps.
            if isempty(strfind(R{1,1}{spk},'XLVDT:'))~=1 && X_ind==0;
                X_ind=1;
                if strcmp(R{1,1}{spk},'XLVDT:')
                    XLVDT=str2num(R{1,1}{spk+1});
                else
                    xbf=R{1,1}{spk};
                    xbf(strfind(xbf,'XLVDT:'):6)=[];
                    XLVDT=str2num(xbf);
                end
            end
            
            if isempty(strfind(R{1,1}{spk},'YLVDT:'))~=1 && Y_ind==0;
                Y_ind=1;
                if strcmp(R{1,1}{spk},'YLVDT:')
                    YLVDT=str2num(R{1,1}{spk+1});
                else
                    ybf=R{1,1}{spk};
                    ybf(strfind(ybf,'YLVDT:'):6)=[];
                    YLVDT=str2num(ybf);
                end
            end
            
        end
        
        %Making sure both of the important paramenters were read for every
        %wave
        if SpcR==0 || length(Flip)==1 || IndexR==0
            n=n-1;
            FCnumber=num2str(n);
            %set(handles.Loading_Text,'String','TROUBLE READING WAVE DATA!!!')
            %drawnow
            continue
        end
        
        %Saving the name of the file without any folder names
        for stringk=length(IBWlist{k}):-1:1
            if strcmp(IBWlist{k}(stringk),'\')
                Cname=IBWlist{k}(stringk+1:length(IBWlist{k})-4);
                break
            end
        end
        
        %Saving all the important bits we use
        if isnan(Wbuff.y(:,3))
            DataFile{n,1}=Wbuff.y(:,1);                 %Piezo, or x data
        else
            DataFile{n,1}=Wbuff.y(:,3);                 %Zsnsr, or x data
        end
        
        
        
        DataFile{n,2}=Wbuff.y(:,2);                 %Defl, or y data
        DataFile{n,3}=str2num(R{1,1}{SpcR});        %Spring Constant
        DataFile{n,4}=Flip(2)+1;                    %Where the wave flips
        DataFile{n,5}=Cname;                        %Name of file
        DataFile{n,6}=XLVDT;                        %X position
        DataFile{n,7}=YLVDT;                        %Y position
        DataFile{n,8}=DataFile{n,2}(Flip(2));       %The deepest indentation
        
        %apperently, some of the files remain open for some reason, despite
        %the fact that the functions explicitly close them.
        fclose all;

    end
    
end
%Making sure not to save a string
LoadingMessage=['Finished Loading ' FCnumber ' Curves'];
set(handles.Loading_Text,'String',LoadingMessage)
FCnumber=n;
drawnow

%Saving the Data File and Important Variables
handles.DataFile=DataFile;
handles.D.FCnumber=FCnumber;
handles.D.currentcurve=1;
handles.D.autorun=0;
handles.D.DoManual=0;

%Display currentcurve value
set(handles.crv_numb_box,'String',num2str(handles.D.currentcurve))
set(handles.crv_num_display,'String',['Curve ' num2str(handles.D.currentcurve) ' of ' num2str(handles.D.FCnumber)])

%Initiating a new record
for k=1:FCnumber
    Record(k).CurveNumber=[];
    Record(k).Force=[];
    Record(k).YoungsMod=[];
    Record(k).Keff=[];
    Record(k).Persistence_L=[];
    Record(k).Contour_L=[];
    Record(k).Steepness=[];
    Record(k).Ldistance=[];
    Record(k).Breaklength=[];
    Record(k).ContactPoint=[];
    
    %the following are variables used in the Manual Mode
    Record(k).mMode=0;
    Record(k).linfx=[];
    Record(k).func=[];
    Record(k).xf=[];
    Record(k).pltend=[];
    Record(k).brkend=[];
    Record(k).brkstrt=[];
    Record(k).grtzero=[];
    Record(k).dstt=[];
    Record(k).peff=[];
    Record(k).mflt=[];
end
handles.Record=Record;

%Update the parameters and plot the first curve
guidata(hObject, handles);
fPlotting
