%Kostyantyn Partola
%FRAME plotting and computation file v1.6

%Bounding the curve within the number of curves currently stored
if handles.D.currentcurve>handles.D.FCnumber
    handles.D.currentcurve=handles.D.FCnumber;
    guidata(hObject, handles);
elseif handles.D.currentcurve<1
    handles.D.currentcurve=1;
    guidata(hObject, handles);
end

%Reading and seperately saving the points of a specific curve as
%Force-Displacement. RET refers to the retraction portion, while EXT refers
%to the extension portion.
RETy=handles.DataFile{handles.D.currentcurve,2}(handles.DataFile{handles.D.currentcurve,4}:length(handles.DataFile{handles.D.currentcurve,2}))*handles.DataFile{handles.D.currentcurve,3};
RETx=handles.DataFile{handles.D.currentcurve,1}(handles.DataFile{handles.D.currentcurve,4}:length(handles.DataFile{handles.D.currentcurve,1}));

EXTy=handles.DataFile{handles.D.currentcurve,2}(1:handles.DataFile{handles.D.currentcurve,4})*handles.DataFile{handles.D.currentcurve,3};
EXTx=handles.DataFile{handles.D.currentcurve,1}(1:handles.DataFile{handles.D.currentcurve,4});

%Manual Mode trumps the automated mode
if handles.Record(handles.D.currentcurve).mMode==1
    fmMode
else


%This is a troubleshooting variable, a value of 0 means a good curve.
trigsame=0;

%This variable builds upon itself to eventually let the user know important
%details about a specific curve
Curvenotes='Curve Notes:\n';

%A counting variable
k=[];

if length(RETy)>10 && length(EXTx)>10
    
    %Making every x value unique, this is a requirement for filtering.
    remove=find(isnan(EXTx));
    EXTx(remove)=[];
    EXTy(remove)=[];
    remove=find(isnan(EXTy));
    EXTx(remove)=[];
    EXTy(remove)=[];
    
    [UEXTx, ia, ic]=unique(EXTx);
    ia=sort(ia);
    
    aaEXTx=EXTx(ia);
    aaEXTy=EXTy(ia);
    EXTx=EXTx(ia);
    EXTy=EXTy(ia);
    
    remove=find(isnan(RETx));
    RETx(remove)=[];
    RETy(remove)=[];
    remove=find(isnan(RETy));
    RETx(remove)=[];
    RETy(remove)=[];
    
    [URETx, ia, ic]=unique(RETx);
    ia=sort(ia);
    
    aaRETx=RETx(ia);
    aaRETy=RETy(ia);
    RETx=RETx(ia);
    RETy=RETy(ia);
    
    filtersize = 16; % filter size , this is the most important parameter for filtering
    filtfiltB = ones(1,filtersize)/filtersize;
    filtfiltA = filtfiltB * 0;
    filtfiltA(1) = 1;
    if length(EXTx)>((16*3)+2) && length(RETx)>((16*3)+2)
        aaRETy = filtfilt(filtfiltB, filtfiltA, aaRETy);
        aaEXTy = filtfilt(filtfiltB, filtfiltA, aaEXTy);
    else
        trigsame=1;
    end
    

    %------------------------%Stiffness Fitting%--------------------------%
    %Curve goodness of fit check, this is seperate from the trigsame
    %variable since it doesn't produce an error when ignored. A value of 1
    %means a good curve.
    Checknrm=1;
    
    %The program scales the data here to improve the fit (the MATLAB least-
    %squares fit terminates its convergence search at very low residual
    %values, thus the extremely low x and y values of the data can cause
    %the fit to terminate on the first step).
    EXTx=EXTx.*10^9;
    EXTy=EXTy.*10^12;

    RETx=RETx.*10^9;
    RETy=RETy.*10^12;
    
    aaEXTx=aaEXTx.*10^9;
    aaEXTy=aaEXTy.*10^12;

    aaRETx=aaRETx.*10^9;
    aaRETy=aaRETy.*10^12;

    %The fit is performed once for flattening and once for Young's Modulus
    %calculation from the Hertz Cone indentation model
    if handles.D.do_stff==1
        %Make a new flattened EXT
        %Fcur=round(length(EXTx)*handles.D.freeairprcnt);
        aaidist=round(length(aaEXTx)/20);
        CPprcntguess=0.5;
        for nk=1:20;
            aaslope(nk)=(EXTy(end-((nk-1)*aaidist))-EXTy(end-(nk*aaidist)))/(EXTx(end-((nk-1)*aaidist))-EXTx(end-(nk*aaidist)));
            if nk>2
                if (aaslope(nk)/aaslope(1))<=0.9
                    CPprcntguess=(length(EXTx)-((nk+1)*aaidist))/length(EXTx);
                    break
                end
            end
        end
        
        Fcur=round(length(EXTx)*CPprcntguess);
        
        p=polyfit(EXTx(1:Fcur),EXTy(1:Fcur),1);
        linfx=@(x)(p(1).*x)+p(2);
        flEXTy=EXTy-linfx(EXTx);
        
        %Find an estimated contact point and limit
        gtE0=find(flEXTy<0);
        if handles.D.cnt_dist_menu==1
            EXT_pltend=find(flEXTy<(max(flEXTy)*handles.D.fitrngprcnt));
        elseif handles.D.cnt_dist_menu==2
            EXT_pltend=find(flEXTy<(max(flEXTy)*0.7));
        end
        pltend=EXT_pltend(end);
        
        %This are unknown variables that are being fit by a least squares
        %curve fit.
        % 1 - Linear initial slope
        % 2 - Linear initial adjustment
        % 3 - Contact Point
        % 4 - Coefficient
        % 5 - Linear initial point
        % 6 - Linear final slope
        
        %This is the initial guess for these values, based on the extension
        %curve
        x0=[p(1) p(2) EXTx(gtE0(end))-5 0.02 EXTx(pltend) 6];
        
        %These are the lower and upper bounds. They have been selected
        %arbitrarily, from practice, but do have a big impact on the fit.
        %EXTx(gtE0(end))-125
        lb=[-Inf -Inf mean([EXTx(Fcur) EXTx(1)]) -Inf EXTx(gtE0(end)) -Inf];
        up=[Inf Inf EXTx(gtE0(end)) Inf EXTx(pltend) Inf];
        
        %Making sure that there is a range in the upper and lower bounds
        for lk=1:6
            if lb(lk)==up(lk)
                trigsame=1;
            end
        end
        
        %Making sure that the extension curve actually deflected up
        if isempty(pltend)
            trigsame=1;
        end
        
        %There are a number of reason why the fit might fail, though almost
        %always when the fit fails, the data is not usable.
        if handles.D.stffmodel==1
            if trigsame==0;
                %This is the function that will be fit
                if handles.D.cnt_dist_menu==1
                    func=@(x,xdata)x(1)*xdata+x(2)+(x(4).*(xdata-x(3)).^(2).*(fHside(x(5)-xdata)-fHside(x(3)-xdata)))+(fHside(xdata-x(5)).*((x(6).*(xdata-x(5)))+(x(4).*((x(5)-x(3)).^2))));
                elseif handles.D.cnt_dist_menu==2
                    func=@(x,xdata)x(1)*xdata+x(2)+(x(4).*(xdata-x(3)).^(2).*(fHside((x(3)+handles.D.fitrngprcnt)-xdata)-fHside(x(3)-xdata)))+(fHside(xdata-(x(3)+handles.D.fitrngprcnt)).*((x(6).*(xdata-(x(3)+handles.D.fitrngprcnt)))+(x(4).*(((x(3)+handles.D.fitrngprcnt)-x(3)).^2))));
                end
                %These commented function are "trial" functions just to test
                %the code. They can be used as foundations to create custom fit
                %functions.
                
                %%%%func=@(x,xdata)x(1)*xdata+x(2)+(fHside(xdata-x(3)).*x(4).*(xdata-x(3)).^(2))+(fHside(xdata-x(5)).*((x(6).*(xdata-x(5)).^(2))));
                %%%%func=@(x,xdata)x(1)*xdata+x(2)+(fHside(xdata-x(3)).*x(4).*(xdata-x(3)).^(2));
                
                
                %The least squares curve fit of the filtered data
                opts = optimset('Display','off','TolFun',1e-8,'TolX',1e-8);   %do not output a writeup
                [xf, resnorm]=lsqcurvefit(func,x0,EXTx,aaEXTy,lb,up,opts);
                if handles.D.cnt_dist_menu==2
                    xf(5)=xf(3)+handles.D.fitrngprcnt;
                end
                parameter = 3/4 * (1/(1-handles.D.neu^2)) * tan(handles.D.theta);
                ContactPoint=xf(3);
                YoungsMod=xf(4)/(parameter*10^(-6));
            else
                if handles.D.filt_stiff==1
                    Checknrm=0;
                    Curvenotes=[Curvenotes 'The Extension Curve does not agree with the stiffness model: all calculations were skipped\n'];
                end
                handles.D.do_stff=0;
                YoungsMod=0;
                ContactPoint=EXTx(1);
            end
        elseif handles.D.stffmodel==2
            if trigsame==0;
                %This is the function that will be fit
                if handles.D.cnt_dist_menu==1
                    func=@(x,xdata)x(1)*xdata+x(2)+(x(4).*((xdata-x(3)).^(3/2)).*(fHside(x(5)-xdata)-fHside(x(3)-xdata)))+(fHside(xdata-x(5)).*((x(6).*(xdata-x(5)))+(x(4).*((x(5)-x(3)).^(3/2)))));
                elseif handles.D.cnt_dist_menu==2
                    func=@(x,xdata)x(1)*xdata+x(2)+(x(4).*((xdata-x(3)).^(3/2)).*(fHside((x(3)+handles.D.fitrngprcnt)-xdata)-fHside(x(3)-xdata)))+(fHside(xdata-(x(3)+handles.D.fitrngprcnt)).*((x(6).*(xdata-(x(3)+handles.D.fitrngprcnt)))+(x(4).*(((x(3)+handles.D.fitrngprcnt)-x(3)).^(3/2)))));
                end
                
                %The least squares curve fit of the filtered data
                opts = optimset('Display','off','TolFun',1e-8,'TolX',1e-8);   %do not output a writeup
                lsqnonlin
                [xf, resnorm]=lsqcurvefit(func,x0,EXTx,aaEXTy,lb,up,opts);
                if handles.D.cnt_dist_menu==2
                    xf(5)=xf(3)+handles.D.fitrngprcnt;
                end
                parameter = 4/3 * (1/(1-handles.D.neu^2)) * sqrt(handles.D.rval);
                ContactPoint=xf(3);
                YoungsMod=xf(4)/(parameter*10^(-6));
            else
                if handles.D.filt_stiff==1
                    Checknrm=0;
                    Curvenotes=[Curvenotes 'The Extension Curve does not agree with the stiffness model: all calculations were skipped\n'];
                end
                handles.D.do_stff=0;
                YoungsMod=0;
                ContactPoint=EXTx(1);
            end
        %PASTE fCustom_model_statement.m HERE    
        end
    else
        YoungsMod=0;
        ContactPoint=EXTx(1);
    end
    
    %This if statement handles flattening of the curve and stiffness if
    %both are enabled, if only stiffness is enabled, the previous fit is
    %the only relevant one.
    if handles.D.fltn==1 && handles.D.do_stff==0
        %If stiffness is not enabled by flattening is, the program will 
        %linearly fit the assigned free air precentage of the extension 
        %curve and substract this fit from the data.
        Fcur=round(length(EXTx)*handles.D.freeairprcnt);
        p=polyfit(EXTx(1:Fcur),EXTy(1:Fcur),1);
        linfx=@(x)(p(1).*x)+p(2);
        handles.Record(handles.D.currentcurve).linfx=p;
        guidata(hObject, handles);
        RETy=RETy-linfx(RETx);
        EXTy=EXTy-linfx(EXTx);
        
        aaEXTy=aaEXTy-linfx(aaEXTx);
        aaRETy=aaRETy-linfx(aaRETx);
    elseif handles.D.fltn==1 && handles.D.do_stff==1
        %When flattening using the lsq fit from the stiffness caclucation,
        %the fit must be done a second time after flattening
        linfx=@(y)(xf(1).*y)+xf(2);
        
        RETy=RETy-linfx(RETx);
        EXTy=EXTy-linfx(EXTx);
        
        aaEXTy=aaEXTy-linfx(aaEXTx);
        aaRETy=aaRETy-linfx(aaRETx);
        handles.Record(handles.D.currentcurve).linfx=xf;
        guidata(hObject, handles);
        %Find an estimated contact point and limit
        gtE0=find(flEXTy<0);
        if handles.D.cnt_dist_menu==1
            EXT_pltend=find(flEXTy<(max(flEXTy)*handles.D.fitrngprcnt));
        elseif handles.D.cnt_dist_menu==2
            EXT_pltend=find(flEXTy<(max(flEXTy)*0.7));
        end
        pltend=EXT_pltend(end);
        
        %This are unknown variables that are being fit by a least squares
        %curve fit.
        % 1 - Linear initial slope
        % 2 - Linear initial adjustment
        % 3 - Contact Point
        % 4 - Coefficient
        % 5 - Linear initial point
        % 6 - Linear final slope
        
        %This is the initial guess for these values, based on the extension
        %curve
        x0=[p(1) p(2) EXTx(gtE0(end))-5 0.02 EXTx(pltend) 6];
        
        %These are the lower and upper bounds. They have been selected
        %arbitrarily, from practice, but do have a big impact on the fit.
        %EXTx(gtE0(end))-125
        lb=[-Inf -Inf mean([EXTx(1) xf(3)]) -Inf EXTx(gtE0(end)) -Inf];
        up=[Inf Inf EXTx(gtE0(end)) Inf EXTx(pltend) Inf];
        
        if handles.D.stffmodel==1
            %This is the function that will be fit
            if handles.D.cnt_dist_menu==1
                func=@(x,xdata)x(1)*xdata+x(2)+(x(4).*(xdata-x(3)).^(2).*(fHside(x(5)-xdata)-fHside(x(3)-xdata)))+(fHside(xdata-x(5)).*((x(6).*(xdata-x(5)))+(x(4).*((x(5)-x(3)).^2))));
            elseif handles.D.cnt_dist_menu==2
                func=@(x,xdata)x(1)*xdata+x(2)+(x(4).*(xdata-x(3)).^(2).*(fHside((x(3)+handles.D.fitrngprcnt)-xdata)-fHside(x(3)-xdata)))+(fHside(xdata-(x(3)+handles.D.fitrngprcnt)).*((x(6).*(xdata-(x(3)+handles.D.fitrngprcnt)))+(x(4).*(((x(3)+handles.D.fitrngprcnt)-x(3)).^2))));
            end
            
            %The least squares curve fit of the filtered data
            opts = optimset('Display','off','TolFun',1e-8,'TolX',1e-8);   %do not output a writeup
            [xf, ~,residual]=lsqcurvefit(func,x0,EXTx,aaEXTy,lb,up,opts);
            if handles.D.cnt_dist_menu==2
                xf(5)=xf(3)+handles.D.fitrngprcnt;
            end
            resnorm=sqrt(mean(residual.^2));
            parameter = 3/4 * (1/(1-handles.D.neu^2)) * tan(handles.D.theta);
            ContactPoint=xf(3);
            YoungsMod=xf(4)/(parameter*10^(-6));
            
            %Check to see if curve fits well, ignore the curve if it doesn't
            if resnorm > handles.D.Stiff_Filt_Threshold && handles.D.filt_stiff==1
                Checknrm=0;
                YoungsMod=0;
                Curvenotes=[Curvenotes 'The Extension Curve does not agree with the stiffness model: all calculations were skipped\n'];
            end
        elseif handles.D.stffmodel==2
            %This is the function that will be fit
            if handles.D.cnt_dist_menu==1
                func=@(x,xdata)x(1)*xdata+x(2)+(x(4).*((xdata-x(3)).^(3/2)).*(fHside(x(5)-xdata)-fHside(x(3)-xdata)))+(fHside(xdata-x(5)).*((x(6).*(xdata-x(5)))+(x(4).*((x(5)-x(3)).^(3/2)))));
            elseif handles.D.cnt_dist_menu==2
                func=@(x,xdata)x(1)*xdata+x(2)+(x(4).*((xdata-x(3)).^(3/2)).*(fHside((x(3)+handles.D.fitrngprcnt)-xdata)-fHside(x(3)-xdata)))+(fHside(xdata-(x(3)+handles.D.fitrngprcnt)).*((x(6).*(xdata-(x(3)+handles.D.fitrngprcnt)))+(x(4).*(((x(3)+handles.D.fitrngprcnt)-x(3)).^(3/2)))));
            end
            
            %The least squares curve fit of the filtered data
            opts = optimset('Display','off','TolFun',1e-8,'TolX',1e-8);   %do not output a writeup
            [xf, ~,residual]=lsqcurvefit(func,x0,EXTx,aaEXTy,lb,up,opts);
            if handles.D.cnt_dist_menu==2
                xf(5)=xf(3)+handles.D.fitrngprcnt;
            end
            resnorm=sqrt(mean(residual.^2));
            parameter = 4/3 * (1/(1-handles.D.neu^2)) * sqrt(handles.D.rval);
            ContactPoint=xf(3);
            YoungsMod=xf(4)/(parameter*10^(-6));
            
            %Check to see if curve fits well, ignore the curve if it doesn't
            if resnorm > handles.D.Stiff_Filt_Threshold && handles.D.filt_stiff==1
                Checknrm=0;
                YoungsMod=0;
                Curvenotes=[Curvenotes 'The Extension Curve does not agree with the stiffness model: all calculations were skipped\n'];
            end
        %PASTE fCustom_model_statement.m HERE
        end
    end
    
    %Bringing the data back to the original coordinates.
    EXTx=EXTx.*10^(-9);
    EXTy=EXTy.*10^(-12);

    RETx=RETx.*10^(-9);
    RETy=RETy.*10^(-12);

    aaEXTx=aaEXTx.*10^-9;
    aaEXTy=aaEXTy.*10^-12;

    aaRETx=aaRETx.*10^-9;
    aaRETy=aaRETy.*10^-12;
    
    %-------------------------%Stiffness Ends%----------------------------%
    
    
    %--------------------%Unbinding Eventss Processing%---------------------%
    %Finding the peaks and troughs throughout the data.
    [~, RETy_peaks]=findpeaks(RETy);
    [~, RETy_trofs]=findpeaks(-RETy);

    if handles.D.do_adhs==1 && Checknrm==1 && trigsame==0 && numel(RETy_peaks)>0
        %First, the program stores the peaks and troughs in a particular
        %order.
        if RETy_peaks(1)<RETy_trofs(1)
            RETy_peaks=RETy_peaks(2:end);
        end
        
        if RETy_peaks(end)<RETy_trofs(end)
            RETy_trofs=RETy_trofs(1:(length(RETy_trofs))-1);
        end
        
        %Only the "free air" portion of the retraction curve is considered
        %when evaluating the noise, in order to avoid breaks from effecting
        %this calculation. The data is subtracted from filtered data if
        %noise sensitivity is used to evaluate potential breaks.
        PnT=sort([RETy_peaks RETy_trofs]);
        haflnth=round(length(PnT)*handles.D.freeairprcnt);
        raw_noise=aaRETy(PnT(haflnth:end))-RETy(PnT(haflnth:end));
        noise_SD=mean(raw_noise.^2);
        noise_amp=sqrt(noise_SD);
        
        if handles.D.Brkdetect_Menu==1
            %cut off value for break based on the noise
            co=noise_amp*handles.D.sensitivity;
        elseif handles.D.Brkdetect_Menu==2
            %exact cut off value for finding a break based on user input
            co=handles.D.sensitivity.*10^-12;
        end
        
        %Initiating the storage variables
        breaks=0;
        L=[];
        brknumber=[];
        Breakforce=[];
        BreakDistance=[];
        brkstart=[];
        brkend=[];
        keff=[];
        perl=[];
        conl=[];
        multiplebreaks=0;
        killpeaks=[];
        killtrofs=[];
        
        
        %Ignoring very tiny drops that can interfer with break detection,
        %this function can be controlled with the maximum combine length
        %variable. If it is set to 0, no breaks will be removed.
        for cb=1:(min([length(RETy_trofs) length(RETy_peaks)])-4)
            Dropval=RETy(RETy_peaks(cb))-RETy(RETy_trofs(cb+1));
            Lnghval=RETx(RETy_peaks(cb))-RETx(RETy_trofs(cb+1));
            if Dropval<(noise_amp/2) && Lnghval<(handles.D.MaxCombineL*10^-9)
                killpeaks=[killpeaks cb];
                killtrofs=[killtrofs (cb+1)];
            end
        end
        
        RETy_peaks(killpeaks)=[];
        RETy_trofs(killtrofs)=[];

        %Merging tiny drops relative to their neighbors, this behaviour is
        %controlled by both the combine lengths and ratios. This
        %functionality is here to make breaks with a small "noise" in the
        %middle are still considered depending on the relative size of the
        %noise. Setting either the combine ratio or length to zero disables
        %this function.
        killpeaks=[];
        killtrofs=[];
        for cb=1:(min([length(RETy_trofs) length(RETy_peaks)])-4)
            RiseBH=RETy(RETy_peaks(cb))-RETy(RETy_trofs(cb));
            RiseIF=RETy(RETy_peaks(cb+1))-RETy(RETy_trofs(cb+1));
            Dropval=RETy(RETy_peaks(cb))-RETy(RETy_trofs(cb+1));
            if Dropval<(RiseBH*handles.D.MaxCombineR) && Dropval<(RiseIF*handles.D.MaxCombineR) %&& Lnghval<0.5e-9
                killpeaks=[killpeaks cb];
                killtrofs=[killtrofs (cb+1)];
            end
        end
        
        RETy_peaks(killpeaks)=[];
        RETy_trofs(killtrofs)=[];
        
        %clear brkstart brkend
        %Find potential Rises
        
        for cb=1:(min([length(RETy_trofs) length(RETy_peaks)])-4)
            Riseval=RETy(RETy_peaks(cb))-RETy(RETy_trofs(cb));
            if Riseval>co
                breaks=breaks+1;
                brknumber(breaks)=cb;
                brkstart(breaks)=RETy_trofs(cb);
                brkend(breaks)=RETy_peaks(cb);
                Breakforce(breaks)=Riseval;
                BreakDistance(breaks)=-RETx(brkend(breaks))+RETx(brkstart(breaks));
            end
        end
        
        
        %-----------------%Unbinding Event Filtering%---------------------%
        if breaks>0
            %The next few steps ensure that each potential unbinding events
            %is not simply noise.
            
            
            %Making sure that every potential unbinding event is steep enough
            killbreaks=[];
            jumpk=[];
            for ES=1:breaks
                jumpk(ES)=(Breakforce(ES)/BreakDistance(ES))*1000;
                if jumpk(ES)<handles.D.jmpminslope;
                    breaks=breaks-1;
                    Curvenotes=[Curvenotes 'Break ignored because it was not steep enough\n'];
                    killbreaks=[killbreaks ES];
                end
            end
            brkstart(killbreaks)=[];
            brknumber(killbreaks)=[];
            brkend(killbreaks)=[];
            Breakforce(killbreaks)=[];
            BreakDistance(killbreaks)=[];
            jumpk(killbreaks)=[];
            
            
            
            killbreaks=[];
            %FRAME first evaluates the behaviour of the data before and
            %after a potential unbinding event using linear approximation 
            %or the mean value if potential unbinding events are 
            %particularly close to one another.
            for ES=1:breaks
                %For every potential break the code searches for nearby
                %potential breaks.
                
                %Seeing if there are breakins in front
                ifc=0;
                brkinfront=0;
                while brkinfront==0
                    ifc=ifc+1;
                    ifdist=abs(RETx(RETy_peaks(brknumber(ES)))-RETx(RETy_trofs(brknumber(ES)+ifc)))*10^9;
                    if ismember(brknumber(ES)+ifc,brknumber) || (brknumber(ES)+ifc)==min([length(RETy_trofs) length(RETy_peaks)])
                        if ifc>1
                            ifc=ifc-1;
                        end
                        brkinfront=1;
                    elseif ifdist>handles.D.keffdist
                        brkinfront=1;
                    end
                end
                
                %Seeing if there are unbinding events behind
                bhc=0;
                brkbehind=0;
                
                behindstart=0;
                
                if (brknumber(ES))>1
                    while brkbehind==0
                        bhc=bhc+1;
                        bhdist=abs(RETx(RETy_trofs(brknumber(ES)-bhc))-RETx(RETy_trofs(brknumber(ES))))*10^9;
                        if ismember(brknumber(ES)-bhc,brknumber) || (brknumber(ES)-bhc)==1
                            if bhc>1
                                bhc=bhc-1;
                            end
                            brkbehind=1;
                        elseif bhdist>handles.D.keffdist
                            brkbehind=1;
                        end
                    end
                    
                else
                    behindstart=1;
                    
                end
                
                
                %Evaluating data tendency after unbinding event
                if behindstart==0
                    IFy=RETy(RETy_peaks(brknumber(ES)):RETy_trofs(brknumber(ES)+ifc));
                else
                    IFy=RETy(1:RETy_trofs(brknumber(ES)+ifc));
                end
                if length(IFy)>6 %ifc>3
                    if behindstart==0
                        IFx=RETx(RETy_peaks(brknumber(ES)):RETy_trofs(brknumber(ES)+ifc));
                    else
                        IFx=RETx(1:RETy_trofs(brknumber(ES)+ifc));
                    end
                    InFront=polyfit(IFx,IFy,1);
                    FrontEval=polyval(InFront,RETx(brkend(ES)));
                else
                    FrontEval=mean(IFy);
                end
                
                %Evaluating data tendency before unbinding event
                BHy=RETy(RETy_trofs(brknumber(ES)-bhc):RETy_trofs(brknumber(ES)));
                if length(BHy)>6 %bhc>3
                    BHx=RETx(RETy_trofs(brknumber(ES)-bhc):RETy_trofs(brknumber(ES)));
                    BeHind=polyfit(BHx,BHy,1);
                    BackEval=polyval(BeHind,RETx(brkstart(ES)));
                else
                    BackEval=mean(BHy);
                end
                
                %Making sure that the data tendency at the unbinding event from
                %either side does, in fact, indicate an unbinding event at least
                %half the size of the cut off and half the size of the
                %potential unbinding event itself.
                Avgchange_one=FrontEval-BackEval;
                if Avgchange_one<(Breakforce(ES)/2) || Avgchange_one<(co/2)
                    breaks=breaks-1;
                    killbreaks=[killbreaks ES];
                end
            end
            
            %This statement is common throughout the code and is used to
            %remove unbinding events that did not pass noise detection from the
            %list of potential unbinding events, or to ignore unbinding events that aren't of
            %interest to the user.
            if isempty(killbreaks)==0
                Curvenotes=[Curvenotes 'Break(s) did not pass noise test\n'];
            end
            
            brkstart(killbreaks)=[];
            brknumber(killbreaks)=[];
            brkend(killbreaks)=[];
            Breakforce(killbreaks)=[];
            BreakDistance(killbreaks)=[];
            jumpk(killbreaks)=[];
            
            %Ignoring unbindign events that are too large
            killbreaks=[];
            for ES=1:breaks
                if Breakforce(ES)>handles.D.toohigh
                    breaks=breaks-1;
                    killbreaks=[killbreaks ES];
                end
            end
            brkstart(killbreaks)=[];
            brknumber(killbreaks)=[];
            brkend(killbreaks)=[];
            Breakforce(killbreaks)=[];
            BreakDistance(killbreaks)=[];
            jumpk(killbreaks)=[];
            
            
            %After this point, it is safe to assume that the u. events that
            %have passed are not noise. The following parameters help
            %filter the data of unwanted u. events, and all of them maybe
            %turned off in the GUI.
            
            
            %Finding the starting point for detachment "Contour Length"
            %there are many option available to the user, so the switch
            %statement for handling different behaviours is quite long.
            %Contact Point Menu
            %1 - Guess via flattening and crossover
            %2 - Use the end of the stiffness model fit as detachment point
            %3 - Use the contact point
            %4 - Use the start of the retraction curve
            %5 - Halfway point from the contact fit point to the extension end
            
            switch handles.D.Contact_Point_Menu
                
                %1 - Guess via flattening and crossover
                case 1
                    if handles.D.fltn==1 && handles.D.do_stff==1
                        grtzero=find(RETy<0);
                        
                    else
                        Fcur=round(length(EXTx)*handles.D.freeairprcnt);
                        p=polyfit(EXTx(1:Fcur),EXTy(1:Fcur),1);
                        linfx=@(x)(p(1).*x)+p(2);
                        cnRETy=RETy-linfx(RETx);
                        grtzero=find(cnRETy<0);
                    end
                    
                    if isempty(grtzero)
                        grtzero=1;
                        Curvenotes=[Curvenotes 'The retraction curve did not cross over the zero threshold using the start of the retraction curve\n'];
                    else
                        grtzero=grtzero(1);
                    end
                    
                %2 - Use the end of the stiffness model fit as "detachment" point    
                case 2
                    if handles.D.do_stff==1
                        grtzero=find(RETx<(xf(5)*10^-9));
                        if numel(grtzero)==0
                            grtzero=1;
                        end
                        grtzero=grtzero(1);
                    else
                        grtzero=1;
                        Curvenotes=[Curvenotes 'Stiffness is disabled, using start of the retraction curve.\n'];
                    end
                    
                %3 - Use the contact point    
                case 3
                    if handles.D.do_stff==1
                        grtzero=find(RETx<(xf(3)*10^-9));
                        if numel(grtzero)==0
                            grtzero=1;
                        end
                        grtzero=grtzero(1);
                    else
                       grtzero=1;
                        Curvenotes=[Curvenotes 'Stiffness is disabled, using start of the retraction curve.\n'];
                    end
                    
                %4 - Use the start of the retraction curve    
                case 4
                    grtzero=1;
                    
                %5 - Halfway point from the contact fit point to the extension end    
                case 5
                    if handles.D.do_stff==1
                        halfwaypnt=mean([RETx(1) xf(3)*10^-9]);
                        grtzero=find(RETx<halfwaypnt);
                        if numel(grtzero)==0
                            grtzero=1;
                        end
                        grtzero=grtzero(1);
                    else
                        grtzero=1;
                        Curvenotes=[Curvenotes 'Stiffness is disabled, using start of the retraction curve.\n'];
                    end
                    
                case 6
                        Fcur=round(length(RETx)*(handles.D.freeairprcnt*0.5));
                        p=polyfit(RETx(end-Fcur:end),RETy(end-Fcur:end),1);
                        linfx=@(x)(p(1).*x)+p(2);
                        YforL=RETy-linfx(RETx);
                        grtzero=find(YforL<0);
                    
                    if isempty(grtzero)
                        grtzero=1;
                        Curvenotes=[Curvenotes 'The retraction curve did not cross over the zero threshold using the start of the retraction curve\n'];
                    else
                        grtzero=grtzero(1);
                    end
                    
                    
            end
            
            %Evaluating each unbinding event's L distance
            for ES=1:breaks
                L(ES)=RETx(grtzero)-RETx(brkstart(ES));
            end
            
            
            %Ignoring if L is too low or too high
            killbreaks=[];
            for ES=1:breaks
                if L(ES)<(handles.D.MinLdist) || L(ES)>handles.D.Llim
                    Curvenotes=[Curvenotes 'An unbinding event is too far from the contact point (or exists before it)\n'];
                    breaks=breaks-1;
                    killbreaks=[killbreaks ES];
                end
            end
            brkstart(killbreaks)=[];
            brknumber(killbreaks)=[];
            brkend(killbreaks)=[];
            L(killbreaks)=[];
            Breakforce(killbreaks)=[];
            BreakDistance(killbreaks)=[];
            jumpk(killbreaks)=[];
            
            %Evaluating the linear tendency of the data before each unbinding event
            %and saving the slope (K-effective), unbinding events can be ignored
            %based on this value. This segment also evaluates the WLC
            %model, if it is selected.
            killbreaks=[];
            dend=[];
            dstt=[];
            peff = [];
            
            %linear regression for K-eff
            if handles.D.keffmodel==1
                for ES=1:breaks
                    dend(ES)=brkstart(ES);
                    found=0;
                    pc=dend(ES);                            %pointcheck
                    
                    %Finding the distance back from each unbinding event that is
                    %either the input distance or the previous unbinding event.
                    while found==0
                        pc=pc-1;
                        if pc==0
                            found=1;
                            dstt(ES)=1;
                            pc=1;
                        elseif ismember(RETx(pc),RETx(RETy_peaks(brknumber)))
                            found=1;
                            dstt(ES)=pc;
                        end
                        dstback=RETx(pc)-RETx(dend(ES));
                        if (dstback*10^9)>handles.D.keffdist
                            found=1;
                            dstt(ES)=pc;
                        end
                    end
                    peff{ES}=polyfit(RETx(dstt(ES):dend(ES)).*(10.^9),RETy(dstt(ES):dend(ES)).*(10.^12),1);
                    keff(ES)=peff{ES}(1);
                    perl(ES)=0;
                    conl(ES)=0;
                    if handles.D.keffprops==1
                    elseif handles.D.keffprops==2 && (keff(ES)<0)
                        breaks=breaks-1;
                        Curvenotes=[Curvenotes 'An unbinding event has an innapropriate k-eff (must be positive)\n'];
                        killbreaks=[killbreaks ES];
                    elseif handles.D.keffprops==3 && (keff(ES)>handles.D.keffmax || keff(ES)<0)
                        breaks=breaks-1;
                        Curvenotes=[Curvenotes 'An unbinding event has an innapropriate k-eff (must be between 0 and 10 pN/nm)\n'];
                        killbreaks=[killbreaks ES];
                    end
                    
                end
                
                
            %WLC model    
            elseif handles.D.keffmodel==2
                for ES=1:breaks
                    dend(ES)=brkstart(ES);
                    found=0;
                    pc=dend(ES);                            %pointcheck
                    
                    %Finding the distance back from each unbinding event that is
                    %either the input distance or the previous unbinding event.
                    while found==0
                        pc=pc-1;
                        if pc==0
                            found=1;
                            dstt(ES)=1;
                            pc=1;
                        elseif ismember(RETx(pc),RETx(RETy_peaks(brknumber))) || pc==grtzero(1)
                            found=1;
                            dstt(ES)=pc+1;
                        end
                        dstback=RETx(pc)-RETx(dend(ES));
                        if (dstback*10^9)>handles.D.wlcdist
                            found=1;
                            dstt(ES)=pc;
                        end
                    end
                    
                    %Data to fit
                    xdfit=(RETx(grtzero(1))-RETx(dstt(ES):dend(ES)))*10^9;
                    fdfit=(RETy(grtzero(1))-RETy(dstt(ES):dend(ES)))*10^12;
                    %Constants
                    kb=1.3806488*10^(-2);
                    T=handles.D.T_box;
                    
                    %Initial guess, and bounds for the persistence length
                    %and countour length, L(1) and L(2) respectively
                    Li=[xdfit(end)/2 xdfit(end)*1.05];
                    Llb=[0 xdfit(end)];
                    Lub=[xdfit(end) 2*xdfit(end)];
                    %Llb=[0 abs(xdfit(end))/2];
                    %Lub=[xdfit(end) abs(xdfit(end))*2];
                    
                    %WLC and curvefit
                    wlcfunc=@(L,xd)((kb.*T)./L(1)).*((1./(4.*(1-(xd./L(2))).^2))-(1./4)+(xd./L(2))); 
                    %wlcfunc=@(L,xd)((kb.*T)./L(1)).*((1./(4.*(1-(xd./(L(2)/2))).^2))-(1./4)+(xd./(L(2)/2))); 
                    opts = optimset('Display','off','TolFun',1e-7,'TolX',1e-7);   %do not output a writeup
                    Lf=lsqcurvefit(wlcfunc,Li,xdfit,fdfit,Llb,Lub,opts);
                    %Lf=lsqcurvefit(wlcfunc,Li,[xdfit+250],fdfit,Llb,Lub,opts);
                    
                    WLCxspace{ES}=linspace(RETx(grtzero(1)),RETx(dstt(ES))).*10^9;
                    WLCfuncspace{ES}=linspace(0,RETx(grtzero(1))-RETx(dstt(ES))).*10^9;
                    WLCresult{ES}=(RETy(grtzero(1)).*(10^12))-wlcfunc(Lf,WLCfuncspace{ES});
                    
                    WLCxspace_2{ES}=linspace(RETx(dstt(ES)),RETx(dend(ES))).*10^9;
                    WLCfuncspace_2{ES}=linspace(RETx(grtzero(1))-RETx(dstt(ES)),RETx(grtzero(1))-RETx(dend(ES))).*10^9;
                    WLCresult_2{ES}=(RETy(grtzero(1)).*(10^12))-wlcfunc(Lf,WLCfuncspace_2{ES});
                    
                    perl(ES)=Lf(1);
                    conl(ES)=Lf(2);
                    
                    
                    %K-Eff calculation and value check
                    distb=handles.D.keffdist;
                    dbcto=WLCxspace_2{ES}(end)+distb;
                    LinLi=WLCxspace_2{ES}<dbcto;
                    
                    keff_xsp=WLCxspace_2{ES}(LinLi);
                    keff_ysp=WLCresult_2{ES}(LinLi);
                    
                    keff(ES)=(keff_ysp(1)-keff_ysp(end))/(keff_xsp(1)-keff_xsp(end));
                    if handles.D.keffprops==1
                    elseif handles.D.keffprops==2 && (keff(ES)<0)
                        breaks=breaks-1;
                        Curvenotes=[Curvenotes 'An unbinding event has an innapropriate k-eff (must be positive)\n'];
                        killbreaks=[killbreaks ES];
                    elseif handles.D.keffprops==3 && (keff(ES)>handles.D.keffmax || keff(ES)<0)
                        breaks=breaks-1;
                        Curvenotes=[Curvenotes 'An unbinding event has an innapropriate k-eff (must be between 0 and 10 pN/nm)\n'];
                        killbreaks=[killbreaks ES];
                    end
                    
                end
            end
            brkstart(killbreaks)=[];
            brkend(killbreaks)=[];
            L(killbreaks)=[];
            Breakforce(killbreaks)=[];
            BreakDistance(killbreaks)=[];
            jumpk(killbreaks)=[];
            peff(killbreaks)=[];
            dend(killbreaks)=[];
            dstt(killbreaks)=[];
            keff(killbreaks)=[];
            perl(killbreaks)=[];
            conl(killbreaks)=[];
        end
        
        
        %Finally, the users have a lot of control over which kind of curves
        %they will save.
        if breaks==0
            FDiff=0;
            Keff=0;
            Persistence_L=0;
            Contour_L=0;
            Steepness=0;
            Ldistance=0;
            Breaklength=0;
            
        elseif handles.D.mbhandle==1 
            %Ignoring curves with multiple unbinding events
            if breaks==1
                FDiff=Breakforce*10^12;
                Keff=keff;
                Persistence_L=perl;
                Contour_L=conl;
                Steepness=jumpk;
                Ldistance=L*10^9;
                Breaklength=BreakDistance*10^9;
            elseif breaks>1
                breaks=0;
                FDiff=0;
                Keff=0;
                Persistence_L=0;
                Contour_L=0;
                Steepness=0;
                Ldistance=0;
                Breaklength=0;
                Curvenotes=[Curvenotes 'Due to user settings multiple unbinding events force the program to ignore this curve\n'];
            end
            
        elseif handles.D.mbhandle==2 
            %Saving the highest force unbinding event
            [FDiff FDi]=max(Breakforce.*10^12);
            Keff=keff(FDi);
            Persistence_L=perl(FDi);
            Contour_L=(FDi);
            Steepness=jumpk(FDi);
            Ldistance=L(FDi)*10^9;
            Breaklength=BreakDistance(FDi)*10^9;
            
        elseif handles.D.mbhandle==3
            %Saving the "total force" or the lowest part of the lowest
            %unbinding event to the highest part of the highest unbinding event.
            FDiff=(RETy(brkend(end))-min(RETy(brkstart)))*10^12;
            if FDiff<(handles.D.toohigh*10^12)
                Keff=keff(1);
                Persistence_L=perl(1);
                Contour_L=conl(1);
                Steepness=jumpk(1);
                Ldistance=L(1)*10^9;
                Breaklength=BreakDistance(1)*10^9;
            else
                FDiff=0;
                Keff=0;
                Persistence_L=0;
                Contour_L=0;
                Steepness=0;
                Ldistance=0;
                Breaklength=0;
                breaks=0;
            end
            
        elseif handles.D.mbhandle==4
            %Saving all unbinding events
            FDiff=Breakforce.*10^12;
            Keff=keff;
            Persistence_L=perl;
            Contour_L=conl;
            Steepness=jumpk;
            Ldistance=L.*10^9;
            Breaklength=BreakDistance.*10^9;
        end
    else
        %This is the behaviour of the code in case unbinding events aren't 
        %calculated, or the curve is somehow inadequate (too short, or too 
        %violently noisy).
        breaks=0;
        FDiff=0;
        Keff=0;
        Persistence_L=0;
        Contour_L=0;
        Steepness=0;
        Ldistance=0;
        Breaklength=0;
    end
    %----------------%unbinding event Filtering ENDS%----------------------%
    
    
    if handles.D.autorun==0
        %This behaviour displays the calculated data but does not save it
        %anywhere. It is meant for user review and is the generally
        %accessible mode of the GUI.
        
        %Converting the curve to plotting coordinates
        EXTx=EXTx.*10^9;
        EXTy=EXTy.*10^12;
        
        RETx=RETx.*10^9;
        RETy=RETy.*10^12;
        
        %Display Curve Notes
        Curvenotes=sprintf(Curvenotes);
        set(handles.CurveNotes_Print,'String',Curvenotes)
        
        %Display Calculated Data
        calcnotes='Calculated Data:\n';
        YoungsMod_str1=['Youngs Modulus (Pa): ' num2str(YoungsMod) '\n'];
        FDiff_str1=['Unbinding Force (pN): ' num2str(FDiff) '\n'];
        Keff_str1=['K-eff (pN/nm): ' num2str(Keff) '\n'];
        Persistence_L_str1=['Persistence Length(nm): ' num2str(Persistence_L) '\n'];
        Contour_L_str1=['Contour Length (nm): ' num2str(Contour_L) '\n'];
        Steepnesss_str1=['Steepness (pN/nm): ' num2str(Steepness) '\n'];
        Ldistance_str1=['L-Distance (nm): ' num2str(Ldistance) '\n'];
        Breaklength_str1=['Unbinding Length (nm): ' num2str(Breaklength) '\n'];
        calcnotes=sprintf([calcnotes YoungsMod_str1 FDiff_str1 Keff_str1 Persistence_L_str1 Contour_L_str1 Steepnesss_str1 Ldistance_str1 Breaklength_str1]);
        set(handles.CurveNotes_display,'String',calcnotes)
        
        %Display Saved Data
        savednotes='Saved Data:\n';
        YoungsMod_str2=['Youngs Modulus (Pa): ' num2str(handles.Record(handles.D.currentcurve).YoungsMod) '\n'];
        FDiff_str2=['Unbinding Force (pN): ' num2str(handles.Record(handles.D.currentcurve).Force) '\n'];
        Keff_str2=['K-eff (pN/nm): ' num2str(handles.Record(handles.D.currentcurve).Keff) '\n'];
        Persistence_L_str2=['Persistence Length(nm): ' num2str(handles.Record(handles.D.currentcurve).Persistence_L) '\n'];
        Contour_L_str2=['Contour Length (nm): ' num2str(handles.Record(handles.D.currentcurve).Contour_L) '\n'];
        Steepnesss_str2=['Steepness (pN/nm): ' num2str(handles.Record(handles.D.currentcurve).Steepness) '\n'];
        Ldistance_str2=['L-Distance (nm): ' num2str(handles.Record(handles.D.currentcurve).Ldistance) '\n'];
        Breaklength_str2=['Unbinding Length (nm): ' num2str(handles.Record(handles.D.currentcurve).Breaklength) '\n'];
        savednotes=sprintf([savednotes YoungsMod_str2 FDiff_str2 Keff_str2 Persistence_L_str2 Contour_L_str2 Steepnesss_str2 Ldistance_str2 Breaklength_str2]);
        set(handles.text18,'String',savednotes)
        
        %Plot Force-Curve and Calculated Data
        axes(handles.axes1)
        plot(RETx,RETy,'b','LineWidth',1)
        hold on
        plot(EXTx,EXTy,'r','LineWidth',1)
        xmax=max([max(RETx) max(EXTx)]);
        xmin=min([min(RETx) min(EXTx)]);
        ymax=max([max(RETy) max(EXTy)]);
        ymin=min([min(RETy) min(EXTy)]);
        axis([xmin xmax ymin ymax])
        
        %Plotting stiffness
        if handles.D.do_stff==1
            xsp=linspace(EXTx(1),max(EXTx),500);
            plot(xsp,func(xf,xsp),'k--','LineWidth',1.5)
            plot(xf(3),func(xf,xf(3)),'kx')
            plot(xf(3),func(xf,xf(3)),'ko')
            plot(xf(5),func(xf,xf(5)),'gx')
            plot(xf(5),func(xf,xf(5)),'go')
        end
        
        %Plotting unbinding events
        if handles.D.do_adhs==1
            for pc=1:breaks
                plot(RETx(brkstart(pc):brkend(pc)),RETy(brkstart(pc):brkend(pc)),'m','LineWidth',1)
                plot(RETx(grtzero(1)),RETy(grtzero(1)),'o','Color',[1 0.35 0])
                plot(RETx(grtzero(1)),RETy(grtzero(1)),'x','Color',[1 0.35 0])
                linkeff=@(x)((peff{pc}(1).*x)+peff{pc}(2));
                if handles.D.keffmodel==1
                    plot(RETx(dstt(pc):dend(pc)),linkeff(RETx(dstt(pc):dend(pc))),'Color',[0.2 0.95 0.65],'LineWidth',1.5)
                elseif handles.D.keffmodel==2
                    plot(WLCxspace{pc},WLCresult{pc},'Color',[0.2 0.95 0.65],'LineStyle','--','LineWidth',1.5)
                    plot(WLCxspace_2{pc},WLCresult_2{pc},'Color',[0.2 0.95 0.65],'LineStyle','-','LineWidth',1.5)
                end
            end
        end
        
        %Setting plotting to familiar handles with the reversed x-direction
        %as well as labeling the axes
        set(gca,'xdir','reverse')
        title([handles.DataFile{handles.D.currentcurve,5}],'Interpreter', 'none');
        xlabel('Z-sensor (nm)')
        ylabel('Force (pN)')
        hold off
    elseif handles.D.autorun==1
        %Automatic saving for autorunning the processing as well as manual
        %saving. This mode saves the data that would normally be
        %displayed. Plotting, however, takes more time, and is generally an
        %inconvenience when automatically processing large ammounts of
        %data.
        
        if (trigsame==0 && handles.D.do_stff==0)
        else
            handles.Record(handles.D.currentcurve).YoungsMod=YoungsMod;
            handles.Record(handles.D.currentcurve).ContactPoint=ContactPoint;
        end
        
        if handles.D.do_adhs==0
        else
            %Clearing up saving procedure when multiple unbinding events are present, but
            %only one value is of interest.
            if breaks<=1 || (handles.D.mbhandle==3 && breaks>1) || (handles.D.mbhandle==2 && breaks>1)
                bk=handles.D.currentcurve;
            else
                bk=[];
                bk(1:breaks)=handles.D.currentcurve;
            end
            
            handles.Record(handles.D.currentcurve).CurveNumber=bk;
            handles.Record(handles.D.currentcurve).Force=FDiff;
            handles.Record(handles.D.currentcurve).Keff=Keff;
            handles.Record(handles.D.currentcurve).Persistence_L=Persistence_L;
            handles.Record(handles.D.currentcurve).Contour_L=Contour_L;
            handles.Record(handles.D.currentcurve).Steepness=Steepness;
            handles.Record(handles.D.currentcurve).Ldistance=Ldistance;
            handles.Record(handles.D.currentcurve).Breaklength=Breaklength;
        end
        guidata(hObject, handles);
    end
else
    handles.Record(handles.D.currentcurve).CurveNumber=handles.D.currentcurve;
    handles.Record(handles.D.currentcurve).YoungsMod=0;
    handles.Record(handles.D.currentcurve).Force=0;
    handles.Record(handles.D.currentcurve).Keff=0;
    handles.Record(handles.D.currentcurve).Persistence_L=0;
    handles.Record(handles.D.currentcurve).Contour_L=0;
    handles.Record(handles.D.currentcurve).Steepness=0;
    handles.Record(handles.D.currentcurve).Ldistance=0;
    handles.Record(handles.D.currentcurve).Breaklength=0;
    handles.Record(handles.D.currentcurve).ContactPoint=0;
    guidata(hObject, handles);
end

if trigsame==1
    handles.D.do_stff=1;
end

end